/*****************************************************************************
*                                                                            *
*                     SOFTING Industrial Automation GmbH                     *
*                          Richard-Reitzner-Allee 6                          *
*                                D-85540 Haar                                *
*                        Phone: ++49-89-4 56 56-0                            *
*                          Fax: ++49-89-4 56 56-3 99                         *
*                                                                            *
*          Copyright (C) SOFTING Industrial Automation GmbH 2000-2010        *
*                             All Rights Reserved                            *
*                                                                            *
******************************************************************************

FILE_NAME               FDLGDL.C

PROJECT_NAME            PROFIBUS

MODULE                  FDLGDL

COMPONENT_LIBRARY       PAPI Lib, PAPICLIENT.LIB
                        PAPI DLL, PAPICLIENT.DLL

AUTHOR                  SOFTING

VERSION                 5.23.0.00.release

DATE                    30-October-2000

STATUS                  finished

FUNCTIONAL_MODULE_DESCRIPTION

This modul contains the FDL(IF)-Service-Specific-Functions which return the length
length of the Request- or Response-Datas.


RELATED_DOCUMENTS
=============================================================================*/
#include "keywords.h"

INCLUDES

#if defined (WIN16) || defined (WIN32)
#include <windows.h>
#endif

#include "pb_type.h"
#include "pb_if.h"
#include "pb_err.h"
#include "pb_fdl.h"

GLOBAL_DEFINES

LOCAL_DEFINES

EXPORT_TYPEDEFS

LOCAL_TYPEDEFS

FUNCTION_DECLARATIONS

EXPORT_DATA

IMPORT_DATA

LOCAL_DATA

#ifdef WIN32
#pragma check_stack (off)
#endif

FUNCTION PUBLIC INT16 fdlgdl_get_data_len
          (
            IN  INT16      result,                        /* Service-Result */
            IN  USIGN8     service,                              /* Service */
            IN  USIGN8     primitive,                  /* Service-Primitive */
            IN  USIGN8 FAR *data_ptr,                    /* pointer to data */
            OUT INT16      *data_len_ptr                  /* length of data */
          )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to return the data length of FDL-SERVICES

possible return values:
- Data-length

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  INT16 ret_val = E_OK;

  FUNCTION_BODY

  *data_len_ptr = 0;

  switch (service)
  {
          case FDLIF_SDA:
          case FDLIF_SDN:
          case FDLIF_SRD:
               {
               T_FDLIF_SDN_SDA_SRD_REQ FAR* req = (T_FDLIF_SDN_SDA_SRD_REQ FAR*) data_ptr;
               *data_len_ptr = sizeof (T_FDLIF_SDN_SDA_SRD_REQ) + req->length;
               }
               break;

          case FDLIF_REPLY_UPDATE:
          case FDLIF_REPLY_UPDATE_MULTIPLE:
               {
               T_FDLIF_RUP_REQ FAR* req = (T_FDLIF_RUP_REQ FAR*) data_ptr;
               *data_len_ptr = sizeof (T_FDLIF_RUP_REQ) + req->length;
               }
               break;

          case FDLIF_SET_BUSPARAMETER:
               *data_len_ptr = sizeof (T_FDLIF_SET_BUSPARAMETER_REQ);
               break;

          case FDLIF_READ_BUSPARAMETER:
               *data_len_ptr = 0;
               break;

          case FDLIF_SAP_ACTIVATE:
               *data_len_ptr = sizeof (T_FDLIF_SAP_ACTIVATE_REQ);
               break;

          case FDLIF_RSAP_ACTIVATE:
               *data_len_ptr = sizeof (T_FDLIF_RSAP_ACTIVATE_REQ);
               break;

          case FDLIF_SAP_CHANGE_ACCESS:
               *data_len_ptr = sizeof (T_FDLIF_SAP_CHANGE_REQ);
               break;

          case FDLIF_SAP_DEACTIVATE:
               *data_len_ptr = sizeof (T_FDLIF_SAP_DEACTIVATE_REQ);
               break;

          case FDLIF_EXIT:
               *data_len_ptr = 0;
               break;

          default:
               *data_len_ptr = 0;
               ret_val = E_IF_INVALID_SERVICE;
               break;

  }
  return(ret_val);
}

#ifdef WIN32
#pragma check_stack
#endif