/******************************************************************************
*                                                                             *
*                      SOFTING Industrial Automation GmbH                     *
*                          Richard-Reitzner-Allee 6                           *
*                                D-85540 Haar                                 *
*                        Phone: (++49)-(0)89-45656-0                          *
*                        Fax:   (++49)-(0)89-45656-399                        *
*                                                                             *
*           Copyright (C) SOFTING Industrial Automation GmbH 1998-2010        *
*                              All Rights Reserved                            *
*                                                                             *
*******************************************************************************

FILE_NAME               PAPI.C

PROJECT_NAME            PROFIBUS

MODULE                  PAPIClient

COMPONENT_LIBRARY       PAPIClient Lib
                        PAPIClient DLL

AUTHOR                  SOFTING

VERSION                 5.45.0.00.release

DATE                    30-November-2010

STATUS                  finished


FUNCTIONAL_MODULE_DESCRIPTION


PROFIBUS DP Master, FMS/DPV1 Master and DP-Slave Application Program Interface (PAPI)
for Windows/Linux using TCP Socket services
-------------------------------------------------------------------------------------

This modul contains the PROFIBUS DP Master, FMS/DPV1 Master and DP-Slave Application Program
Interface functions.


RELATED_DOCUMENTS
=============================================================================*/
#include "keywords.h"

INCLUDES

#include <stdio.h>

#ifdef WIN32
#include <windows.h>
#endif

#ifdef LINUX
#include <string.h>
#endif

#include "pb_type.h"
#include "pb_err.h"
#include "pb_if.h"
#include "pb_fms.h"
#include "pb_fm7.h"
#include "pb_fmb.h"

#include "papisock.h"


/****************************************************************************/

GLOBAL_DEFINES

/****************************************************************************/

LOCAL_DEFINES

#ifdef WIN32
#define ENTER_CRITICAL_SECTION()  EnterCriticalSection(&CriticalSection)
#define LEAVE_CRITICAL_SECTION()  LeaveCriticalSection(&CriticalSection)
#elif defined LINUX
#define ENTER_CRITICAL_SECTION()
#define LEAVE_CRITICAL_SECTION()
#else
#error no valid operating system defined !!!!
#endif

/****************************************************************************/

EXPORT_TYPEDEFS

/****************************************************************************/

LOCAL_TYPEDEFS

/****************************************************************************/

FUNCTION_DECLARATIONS

extern INT16   dpgdl_get_data_len   (INT16,USIGN8,USIGN8,USIGN8*,INT16*);
extern INT16   dpsgdl_get_data_len  (INT16,USIGN8,USIGN8,USIGN8*,INT16*);
extern INT16   fdlgdl_get_data_len  (INT16,USIGN8,USIGN8,USIGN8*,INT16*);
extern INT16   fm7gdl_get_data_len  (INT16,USIGN8,USIGN8,USIGN8*,INT16*);
extern INT16   fmsgdl_get_data_len  (INT16,USIGN8,USIGN8,USIGN8*,INT16*);
extern INT16   fmbgdl_get_data_len  (INT16,USIGN8,USIGN8,USIGN8*,INT16*);

extern VOID    profile_open         (VOID);
extern VOID    profile_close        (VOID);
extern BOOL    profile_read         (const char*);
extern int     profile_get_int      (const char*,const char*,int);
extern size_t  profile_get_string   (const char*,const char*,char*);


/****************************************************************************/

EXPORT_DATA

/****************************************************************************/

IMPORT_DATA

/****************************************************************************/

LOCAL_DATA

#ifdef WIN32
static const char copyright[] = "PROFIBUS Windows Client-Socket API (c) Copyright 2008-2010. SOFTING Industrial Automation GmbH. All Rights Reserved.";
#elif defined LINUX
static const char copyright[] = "PROFIBUS Linux Client-Socket API (c) Copyright 2008-2010. SOFTING Industrial Automation GmbH. All Rights Reserved.";
#else
#error no operating system defined !!!
#endif


static SOCKET           Socket;
static USIGN16          FrameNumber;
static BOOL             bPapiInitialized = FALSE;

#ifdef WIN32
static CRITICAL_SECTION CriticalSection;
#endif

#ifdef WIN32
#pragma check_stack (off)
#endif


static const char CONFIG_FILE[] = "papiclient.conf";


// ***************************************************************************
// * FUNCTIONs to convert data from LITTLE ENDIAN to BIG-ENDIAN format
// ***************************************************************************


FUNCTION PUBLIC USIGN16 swap_16_intel_motorola
         (
           IN USIGN16  input16   /* value to swap */
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to swap a INTEGER16 or USIGN16 value from LITTLE ENDIAN to
BIG-ENDIAN format.

IN:  input16  -> 16-Bit value to swap

return value:
- swapped INTEGER16 or USIGN16 value
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  USIGN16 inp16;

  FUNCTION_BODY

  ((USIGN8*) &inp16) [0] = ((USIGN8*) &input16) [1];
  ((USIGN8*) &inp16) [1] = ((USIGN8*) &input16) [0];

  return (inp16);

}

/****************************************************************************/


FUNCTION PUBLIC USIGN32 swap_32_intel_motorola
         (
           IN USIGN32  input32   /* value to swap */
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to swap a INTEGER32 or USIGN32 value from LITTLE ENDIAN to
BIG-ENDIAN format.

IN:  input32  -> 32-Bit value to swap

return value:
- swapped INTEGER32 or USIGN32 value
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  USIGN32 inp32;

  FUNCTION_BODY

  ((USIGN8*) &inp32) [0] = ((USIGN8*) &input32) [3];
  ((USIGN8*) &inp32) [1] = ((USIGN8*) &input32) [2];
  ((USIGN8*) &inp32) [2] = ((USIGN8*) &input32) [1];
  ((USIGN8*) &inp32) [3] = ((USIGN8*) &input32) [0];

  return (inp32);

}

/****************************************************************************/

FUNCTION LOCAL VOID read_config_file
  (
    IN  const char*  ConfigFileName,
    IN  USIGN8       Board,
    OUT char*        pFgName,
    OUT int*         pPort,
    OUT USIGN8*      pBusConnector,
    OUT USIGN32*     pWatchdogTimeout
  )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to read configuration data from configuration-ini-file

Possible return values:
- E_OK

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  char GroupString[20];

  FUNCTION_BODY

  profile_open();

  if (!profile_read(ConfigFileName))
  {
    strcpy(pFgName,"192.168.212.212");         /* default FGxxxPB IP address */
    *pPort            = 2356;                           /* PAPI-Servers port */
    *pBusConnector    = 0;
    *pWatchdogTimeout = 10000;
    profile_close();
    return;
  }

  sprintf(GroupString,"Board%d",Board);

  if (profile_get_string(GroupString, "FG-IpAddress",pFgName) == 0)
  {
    /* set default address ------------------------------------------------- */
    strcpy(pFgName,"192.168.212.212");
  }

  *pPort            = profile_get_int(GroupString, "PAPI-Port",2356);
  *pBusConnector    = profile_get_int(GroupString, "BusConnector",0);
  *pWatchdogTimeout = profile_get_int(GroupString, "WatchdogTimeout",10000);

  profile_close();

  return;
}


/****************************************************************************/


FUNCTION GLOBAL INT16 CALL_CONV profi_init
         (
           IN USIGN8   Board,
           IN USIGN32  ReadTimeout,
           IN USIGN32  WriteTimeout
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to initialize the WIN32 PROFIBUS-API. The function function
has to be called before any other function of PROFIBUS-API is called.

IN:  Board          -> Fg number (0..255)
IN:  ReadTimeout    -> receive timeout in ms (WAIT_FOREVER for infinite wait)
IN:  Writetimeout   -> send timeout in ms (WAIT_FOREVER for infinite wait)


Possible return values:
- E_OK                             -> Interface is initialized
- E_IF_NO_CNTRL_RES                -> controller does not respond
- E_IF_INVALID_CNTRL_TYPE_VERSION  -> invalid controller type or software version
- E_IF_INIT_INVALID_PARAMETER      -> invalid initialize parameter
- E_IF_INVALID_VERSION             -> invalid version
- E_IF_NO_CNTRL_PRESENT            -> no controller present
- E_IF_INIT_FAILED                 -> initialization failed
- E_IF_SOCKET_ERROR                -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  USIGN32          WatchdogTimeout;                   /* Timeout in ms units */
  USIGN8           BusConnector;
  int              Offset, i;
  int              Port;
  char             FgName[20];

  FUNCTION_BODY

  Socket      = 0;
  FrameNumber = 0;

  ENTER_CRITICAL_SECTION ();

  if (bPapiInitialized)
  {
    return(E_OK);
  }

  /* get configuration paramter to access the FG -----------------------------------*/
  read_config_file(CONFIG_FILE,Board,FgName,&Port,&BusConnector,&WatchdogTimeout);

  /* initialize the sockets --------------------------------------------------------*/
  RetVal = socket_init(FgName,Port,&Socket);
  if (RetVal != E_OK)
  {
    LEAVE_CRITICAL_SECTION ();
    return(E_IF_INIT_FAILED);
  }

  /* send START message ------------------------------------------------------------*/
  /* build message header ----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"start",++FrameNumber);

  /* send message ----------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,FALSE);
  if (RetVal != E_OK)
  {
    LEAVE_CRITICAL_SECTION ();
    return(RetVal);
  }

  /* send prof_init() message ------------------------------------------------------*/
  /* build message header ----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"profi_init",++FrameNumber);

  /* set message data --------------------------------------------------------------*/
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(BusConnector);
  SocketMessage.Header.para_len[i++] = sizeof(ReadTimeout);
  SocketMessage.Header.para_len[i++] = sizeof(WriteTimeout);
  SocketMessage.Header.para_len[i++] = sizeof(WatchdogTimeout);

  Offset = 0; i = 0;
  *((USIGN8*)  &SocketMessage.Id.Data[Offset]) = BusConnector;
  Offset += SocketMessage.Header.para_len[i++];
  *((USIGN32*) &SocketMessage.Id.Data[Offset]) = ReadTimeout;
  Offset += SocketMessage.Header.para_len[i++];
  *((USIGN32*) &SocketMessage.Id.Data[Offset]) = WriteTimeout;
  Offset += SocketMessage.Header.para_len[i++];
  *((USIGN32*) &SocketMessage.Id.Data[Offset]) = WatchdogTimeout;

  /* send message ----------------------------------------------------------------- */
  RetVal = socket_send_data(Socket, &SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    bPapiInitialized = TRUE;
  }
  else
  {
    socket_close(Socket);
  }

  LEAVE_CRITICAL_SECTION ();

  return(RetVal);
}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_end(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to shutdown the PROFIBUS-API and shutdown the PROFIBUS
controller.

Possible return values:
- E_OK
- E_IF_PAPI_NOT_INITIALIZED
- E_IF_SOCKET_ERROR
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;

  FUNCTION_BODY


  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_END message --------------------------------------------------------*/
  /* build message header ----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"profi_end",++FrameNumber);

  /* send message ----------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if(RetVal == E_OK)
  {
    /* send STOP message -----------------------------------------------------------*/
    /* build message header --------------------------------------------------------*/
    socket_build_request_message_header(&SocketMessage,"stop",++FrameNumber);

    /* send message --------------------------------------------------------------- */
    RetVal = socket_send_data(Socket,&SocketMessage,FALSE);
  }

  socket_close (Socket);

  LEAVE_CRITICAL_SECTION();

  return(RetVal);

}

/****************************************************************************/


FUNCTION GLOBAL INT16 CALL_CONV profi_snd_idle(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to send a IDLE-MESSAGE (live sign) to the FG.
controller.

Possible return values:
- E_OK
- E_IF_PAPI_NOT_INITIALIZED      -> API not initialized
- E_IF_SOCKET_ERROR              -> TCP socket error
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal = E_OK;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_IDLE message -------------------------------------------------------*/
  /* build message header ----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"profi_snd_idle",++FrameNumber);

  /* send message ----------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_snd_req_res
         (
          IN T_PROFI_SERVICE_DESCR * pSdb,
          IN VOID                  * pData,
          IN PB_BOOL                 dummy
         )

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to send a Service-Request or a Service-Response
to the FG's PROFIBUS controller.

IN:  pSdb                  -> pointer to SERVICE-DESCRIPTION-BLOCK
IN:  pData                 -> pointer to service specific data
IN:  dummy                 -> dummy


Possible return values:

- E_OK                           -> no error occured

- E_IF_INVALID_LAYER             -> invalid layer
- E_IF_INVALID_SERVICE           -> invalid service identifier
- E_IF_INVALID_PRIMITIVE         -> invalid service primitive
- E_IF_RESOURCE_UNAVAILABLE      -> no resource available
- E_IF_NO_PARALLEL_SERVICES      -> no parallel services allowed
- E_IF_SERVICE_CONSTR_CONFLICT   -> service temporarily not executable
- E_IF_SERVICE_NOT_SUPPORTED     -> service not supported in subset
- E_IF_SERVICE_NOT_EXECUTABLE    -> service not executable

- E_IF_NO_CNTRL_RES              -> controller does not respond  (CMI_TIMEOUT)
- E_IF_INVALID_DATA_SIZE         -> not enough CMI memory available for REQ or RES
- E_IF_INVALID_VERSION           -> invalid version (only PROFDI-IF)
- E_IF_INVALID_PARAMETER         -> invalid parameter in REQ or RES
- E_IF_PAPI_NOT_INITIALIZED      -> API not initialized
- E_IF_SOCKET_ERROR              -> TCP socket error

----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal = E_OK;
  USIGN16          DataLength;
  int              Offset, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if (pSdb == NULL)
  {
    return(E_IF_INVALID_PARAMETER);
  }

  pSdb->comm_ref &= 0x00FF;                         /* clear all BITs in upper byte */

  /* check Service-Primitive ------------------------------------------------------ */
  if ((pSdb->primitive == IND) || (pSdb->primitive == CON))
  {
    return (E_IF_INVALID_PRIMITIVE);
  }

  /* set the result parameter to POS if a request is send -------------------------- */
  if (pSdb->primitive == REQ) pSdb->result = POS;


  /* get service specific data length ---------------------------------------------- */
  switch(pSdb->layer)
  {
    case FMS:
    {
      RetVal = fmsgdl_get_data_len(pSdb->result,pSdb->service,pSdb->primitive,pData,&DataLength);
      break;
    }
    case FM7:
    {
      RetVal = fm7gdl_get_data_len(pSdb->result,pSdb->service,pSdb->primitive,pData,&DataLength);
      break;
    }
    case FDLIF:
    {
      RetVal = fdlgdl_get_data_len(pSdb->result,pSdb->service,pSdb->primitive,pData,&DataLength);
      break;
    }
    case DP:
    {
      RetVal = dpgdl_get_data_len(pSdb->result,pSdb->service,pSdb->primitive,pData,&DataLength);
      break;
    }
    case FMB:
    {
      RetVal = fmbgdl_get_data_len(pSdb->result,pSdb->service,pSdb->primitive,pData,&DataLength);
      break;
    }
    case DPS:
    {
      RetVal = dpsgdl_get_data_len(pSdb->result,pSdb->service,pSdb->primitive,pData,&DataLength);
      break;
    }
    default:
    {
      RetVal = E_IF_INVALID_LAYER;
      break;
    }
  }

  if (RetVal)
  {
    return(RetVal);
  }

  /* check max data length ---------------------------------------------------------*/
  if (DataLength > MAX_PB_DATA_SIZE)
  {
    return(E_IF_INVALID_DATA_SIZE);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_SND_REQ_RES message ------------------------------------------------*/
  /* build message header ----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"profi_snd_req_res",++FrameNumber);

  /* set message data --------------------------------------------------------------*/
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(*pSdb);
  SocketMessage.Header.para_len[i++] = DataLength;

  Offset = 0; i = 0;
  *((T_PROFI_SERVICE_DESCR*) &SocketMessage.Id.Data[Offset]) = *pSdb;

  if (pData != NULL )
  {
    Offset += SocketMessage.Header.para_len[i++];
    memcpy(&SocketMessage.Id.Data[Offset], pData, DataLength);
  }

  /* send message ----------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/


FUNCTION GLOBAL INT16 CALL_CONV profi_rcv_con_ind
         (
          OUT   T_PROFI_SERVICE_DESCR * pSdb,
          OUT   VOID                  * pData,
          INOUT USIGN16               * pDataLength
         )

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to get a Service-Indication or a Service-Confirmation
from the FG's PROFIBUS controller.


OUT:    pSdb             -> pointer to SERVICE-DESCRIPTION-BLOCK
OUT:    pData            -> pointer to data buffer
INOUT:  pDataLength      -> IN:   length of data buffer
                            OUT:  length of received data
Possible return values:

- CON_IND_RECEIVED          -> a confirmation or indication has been received
- NO_CON_IND_RECEIVED       -> no confirmation or indication has been received

- E_IF_FATAL_ERROR          -> unrecoverable error in PROTOCOL SW
- E_IF_INVALID_DATA_SIZE    -> size of data block provided not sufficient
- E_IF_PAPI_NOT_INITIALIZED -> API not initialized
- E_IF_SOCKET_ERROR         -> TCP socket error
----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  int              BytesRead, Offset, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ----------------------------------------------------------*/
  if( (pSdb == NULL) || (pData == NULL) || (pDataLength == NULL) || (*pDataLength > MAX_PB_DATA_SIZE) )
  {
    return(E_IF_INVALID_PARAMETER);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_RCV_CON_IND message -------------------------------------------------*/
  /* build message header -----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"profi_rcv_con_ind",++FrameNumber);

  /* set message data ---------------------------------------------------------------*/
  SocketMessage.Header.para_len[0]        = sizeof(*pDataLength);
  *((USIGN16*) &SocketMessage.Id.Data[0]) = *pDataLength;

  /* send message -------------------------------------------------------------------*/
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);


  if ((RetVal == CON_IND_RECEIVED) || (RetVal == E_IF_FATAL_ERROR))
  {
    /* an indication, confirmation or a fatal error is received ---------------------*/
    Offset = GET_DATA_OFFSET();
    i      = GET_PARA_LEN_OFFSET();
    *pSdb  = *((T_PROFI_SERVICE_DESCR*) &SocketMessage.Id.Data[Offset]);

    Offset      += SocketMessage.Header.para_len[i++];
    BytesRead    = *((USIGN16*) &SocketMessage.Id.Data[Offset]);

    if (BytesRead > *pDataLength) BytesRead = *pDataLength;

    if (BytesRead > 0)
    {
      Offset += SocketMessage.Header.para_len[i];
      memcpy(pData,&SocketMessage.Id.Data[Offset], BytesRead);
    }

    *pDataLength = BytesRead;

    if (pSdb->layer == FMB_USR && pSdb->service == FMB_EXCEPTION && pSdb->primitive == IND)
    {
       RetVal = E_IF_FATAL_ERROR;
    }
  }

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_set_data
        (
          IN  USIGN8    DataId,
          IN  USIGN16   Offset,
          IN  USIGN16   DataLength,
          IN  VOID    * pData
        )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to write DP slve output data to FG's DP_SLAVE_IO_IMAGE

IN:   DataId         -> data image identifier
                         - ID_DP_SLAVE_IO_IMAGE     (DPM: image for slave io data)

IN:   Offset         -> write at offset in data image
IN:   DataLength     -> length of  data to write
IN:   pData          -> pointer to data to write


possible return values:
- E_OK                          -> OK
- E_IF_SERVICE_CONSTR_CONFLICT  -> service not executable at time
- E_IF_SERVICE_NOT_SUPPORTED    -> service not supported
- E_IF_INVALID_DATA_SIZE        -> invalid user data size
- E_IF_PAPI_NOT_INITIALIZED     -> API not initialized
- E_IF_SOCKET_ERROR             -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  int              Offs, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if ((pData == NULL ) || (DataLength > MAX_PB_DATA_SIZE))
  {
    return(E_IF_INVALID_DATA_SIZE);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_SET_DATA message ---------------------------------------------------*/
  /* build message header ----------------------------------------------------------*/
  socket_build_request_message_header(&SocketMessage,"profi_set_data",++FrameNumber);

  /* set message data --------------------------------------------------------------*/
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(DataId);
  SocketMessage.Header.para_len[i++] = sizeof(Offset);
  SocketMessage.Header.para_len[i++] = sizeof(DataLength);
  SocketMessage.Header.para_len[i++] = DataLength;

  Offs = 0; i = 0;
  *((USIGN8*)  &SocketMessage.Id.Data[Offs]) = DataId;

  Offs += SocketMessage.Header.para_len[i++];
  *((USIGN16*) &SocketMessage.Id.Data[Offs]) = Offset;

  Offs += SocketMessage.Header.para_len[i++];
  *((USIGN16*) &SocketMessage.Id.Data[Offs]) = DataLength;

  Offs += SocketMessage.Header.para_len[i++];
  memcpy(&SocketMessage.Id.Data[Offs], pData, DataLength);

  /* send message ------------------------------------------------------------------*/
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}

/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_get_data
        (
          IN     USIGN8     DataId,
          IN     USIGN16    Offset,
          INOUT  USIGN16  * pDataLength,
          OUT    VOID     * pData
        )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to read DP input data from FG's DP_SLVAE_IO_IMAGE

IN:    DataId        -> data image identifier
                         - ID_DP_SLAVE_IO_IMAGE        (DPM: image for slave io data)

IN:    Offset        -> read at offset in data image
INOUT: pDataLength      IN:  length of data buffer
                        OUT: length of received data
OUT:   pData         -> pointer to data buffer

possible return values:
- E_OK                          -> OK
- E_IF_SERVICE_CONSTR_CONFLICT  -> service not executable at time
- E_IF_SERVICE_NOT_SUPPORTED    -> service not supported
- E_IF_INVALID_DATA_SIZE        -> invalid user data size
- E_IF_PAPI_NOT_INITIALIZED     -> API not initialized
- E_IF_SOCKET_ERROR             -> TCP socket error
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  USIGN16          BytesRead;
  int              Offs, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if ((pDataLength == NULL) || (pData == NULL) || (*pDataLength > MAX_PB_DATA_SIZE))
  {
    return(E_IF_INVALID_DATA_SIZE);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_GET_DATA message ---------------------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_get_data",++FrameNumber);

  /* set message data ------------------------------------------------------------- */
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(DataId);
  SocketMessage.Header.para_len[i++] = sizeof(Offset);
  SocketMessage.Header.para_len[i++] = sizeof(*pDataLength);

  Offs = 0; i = 0;
  *((USIGN8*)  &SocketMessage.Id.Data[Offs]) = DataId;

  Offs += SocketMessage.Header.para_len[i++];
  *((USIGN16*) &SocketMessage.Id.Data[Offs]) = Offset;

  Offs += SocketMessage.Header.para_len[i++];
  *((USIGN16*) &SocketMessage.Id.Data[Offs]) = *pDataLength;

  /* send message ---------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    Offs = GET_DATA_OFFSET();
    i    = GET_PARA_LEN_OFFSET();

    BytesRead = *((USIGN16*) &SocketMessage.Id.Data[Offs]);
    if (BytesRead >  *pDataLength) BytesRead = *pDataLength;

    if (BytesRead > 0)
    {
      Offset += (Offs + SocketMessage.Header.para_len[i]);
      memcpy(pData,&SocketMessage.Id.Data[Offset], BytesRead);
    }

    *pDataLength = BytesRead;

  }

  LEAVE_CRITICAL_SECTION();

  return(RetVal);

}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_set_dps_input_data
        (
          IN    USIGN8  * pData,
          IN    USIGN8    DataLength,
          OUT   USIGN8  * pState
        )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to write DP Slave input data to FG's DPS_SLAVE_INPUT_IMAGE

IN:   pData      -> pointer to new input data
IN:   DataLength -> number bytes of input data
OUT:  pState     -> pointer to a status variable for the recent input data status

possible return values:
- E_OK                          -> OK
- E_IF_INVALID_PARAMETER        -> invalid user parameter
- E_IF_NO_CNTRL_RES             -> timeout controller does not response
- E_IF_PAPI_NOT_INITIALIZED     -> API not initialized
- E_IF_SOCKET_ERROR             -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  int              Offset, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if ((DataLength > MAX_PB_DATA_SIZE) || (pData == NULL) || (pState == NULL))
  {
    return(E_IF_INVALID_PARAMETER);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_SET_DPS_INPUT_DATA message ------------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_set_dps_input_data",++FrameNumber);

  /* set message data --------------------------------------------------------------*/
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(DataLength);
  SocketMessage.Header.para_len[i++] = DataLength;

  Offset = 0; i = 0;
  *((USIGN8*) &SocketMessage.Id.Data[Offset]) = DataLength;

  Offset += SocketMessage.Header.para_len[i++];
  memcpy(&SocketMessage.Id.Data[Offset], pData, DataLength);

  /* send message ---------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    Offset = GET_DATA_OFFSET();

    /* get state ----------------------------------------------------------------- */
    *pState = *((USIGN8*) &SocketMessage.Id.Data[Offset]);
  }

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_get_dps_input_data
        (
          OUT   USIGN8 * pData,
          INOUT USIGN8 * pDataLength,
          OUT   USIGN8 * pState
        )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to read back data from FG's DPS_SLAVE_INPUT_IMAGE

OUT:    pData       -> destination buffer for input data
INOUT:  pDataLength -> sizeof destination buffer and number of bytes read
OUT:    pState      -> status of input data if read successfully

possible return values:
- E_OK                          -> OK
- E_IF_INVALID_PARAMETER        -> invalid user parameter
- E_IF_PAPI_NOT_INITIALIZED     -> API not initialized
- E_IF_SOCKET_ERROR             -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  USIGN8           BytesRead;
  int              Offset, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if ((pDataLength == NULL) || (pData == NULL) || (pState == NULL))
  {
    return(E_IF_INVALID_PARAMETER);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_GET_DPS_INPUT_DATA message ------------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_get_dps_input_data",++FrameNumber);

  /* set message data ------------------------------------------------------------- */
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(*pDataLength);

  Offset = 0;
  *((USIGN8*)  &SocketMessage.Id.Data[Offset]) = *pDataLength;

  /* send message ---------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    Offset = GET_DATA_OFFSET();
    i      = GET_PARA_LEN_OFFSET();

    /* get real data length ------------------------------------------------------ */
    BytesRead = *((USIGN8*) &SocketMessage.Id.Data[Offset]);

    /* get back input data ------------------------------------------------------- */
    if (BytesRead >  *pDataLength) BytesRead = *pDataLength;

    if (BytesRead > 0)
    {
      Offset += SocketMessage.Header.para_len[i];
      memcpy(pData, &SocketMessage.Id.Data[Offset], BytesRead);
    }
    i++;
    *pDataLength = BytesRead;

    /* get state ----------------------------------------------------------------- */
    Offset += SocketMessage.Header.para_len[i];
    *pState = *((USIGN8*) &SocketMessage.Id.Data[Offset]);
  }

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/


FUNCTION GLOBAL INT16 CALL_CONV profi_get_dps_output_data
        (
          OUT   USIGN8 * pData,
          INOUT USIGN8 * pDataLength,
          OUT   USIGN8 * pState
        )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to read data from FG's DPS_SLAVE_OUTPUT_IMAGE.

OUT:    pData       -> destination buffer for output data
INOUT:  pDataLength -> sizeof destination buffer and number of bytes read
OUT:    pState      -> status of output data if read successfully

possible return values:
- E_OK                          -> OK
- E_IF_SERVICE_NOT_SUPPORTED    -> service not supported
- E_IF_SERVICE_CONSTR_CONFLICT  -> service not executable at time
- E_IF_INVALID_PARAMETER        -> invalid user parameter
- E_IF_PAPI_NOT_INITIALIZED     -> API not initialized
- E_IF_SOCKET_ERROR             -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  USIGN8           BytesRead;
  int              Offset, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if ((pDataLength == NULL) || (pData == NULL) || (pState == NULL))
  {
    return(E_IF_INVALID_PARAMETER);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_GET_DPS_OUTPUT_DATA message ----------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_get_dps_output_data",++FrameNumber);

  /* set message data ------------------------------------------------------------- */
  i = 0;
  SocketMessage.Header.para_len[i++] = sizeof(*pDataLength);

  Offset = 0;
  *((USIGN8*)  &SocketMessage.Id.Data[Offset]) = *pDataLength;

  /* send message ---------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    Offset = GET_DATA_OFFSET();
    i      = GET_PARA_LEN_OFFSET();

    /* get real data length ------------------------------------------------------ */
    BytesRead = *((USIGN8*) &SocketMessage.Id.Data[Offset]);

    if (BytesRead >  *pDataLength) BytesRead = *pDataLength;

    /* get output data ----------------------------------------------------------- */
    if (BytesRead > 0)
    {
      Offset += SocketMessage.Header.para_len[i];
      memcpy(pData, &SocketMessage.Id.Data[Offset], BytesRead);
    }
    i++;
    *pDataLength = BytesRead;

    /* get state ----------------------------------------------------------------- */
    Offset += SocketMessage.Header.para_len[i];
    *pState = *((USIGN8*) &SocketMessage.Id.Data[Offset]);
  }

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/


FUNCTION GLOBAL INT16 CALL_CONV profi_get_versions
         (
           OUT CSTRING * pPapiVersion,
           OUT CSTRING * pFirmwareVersion
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to get the version strings of FG's PAPI and PROFIBUS firmware.

OUT:    pPapiVersion      -> data buffer for PAPI version string
OUT:    pFirmwareVersion  -> data buffer for PROFIBUS firmware version string

NOTE: There must be at least 100 Bytes (VERSION_STRING_LENGTH) free space for
      each version buffer.

Possible return values:
- E_OK
- E_IF_PAPI_NOT_INITIALIZED      -> API not initialized
- E_IF_INVALID_PARAMETER         -> invalid user parameter
- E_IF_SOCKET_ERROR              -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  int              Offset, i;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if ((pPapiVersion == NULL ) || (pFirmwareVersion == NULL))
  {
    return(E_IF_INVALID_PARAMETER);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_GET_VERSIONS message -----------------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_get_versions",++FrameNumber);

  /* send message ---------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    /* get remote PAPI version ---------------------------------------------------- */
    Offset = GET_DATA_OFFSET();
    i      = GET_PARA_LEN_OFFSET();

    if (   (SocketMessage.Header.para_len[i]          <= VERSION_STRING_LENGTH)
        && (strlen(&SocketMessage.Id.Data[Offset])    <= VERSION_STRING_LENGTH) )
    {
      strcpy(pPapiVersion, &SocketMessage.Id.Data[Offset]);
    }

    /* get remote firmware version ----------------------------------------------- */
    Offset += SocketMessage.Header.para_len[i++];
    if (   (SocketMessage.Header.para_len[i]          <= VERSION_STRING_LENGTH)
        && (strlen(&SocketMessage.Id.Data[Offset])    <= VERSION_STRING_LENGTH) )
    {
       strcpy(pFirmwareVersion, &SocketMessage.Id.Data[Offset]);
    }

    /* convert \n to ' ' -------------------------------------------------------- */
    for (i=0; i < VERSION_STRING_LENGTH; i++)
    {
      if      (pFirmwareVersion[i] == '\n') pFirmwareVersion[i] = ' ';
      else if (pFirmwareVersion[i] == '\0') break;
      else                                  continue;
    }
  }

  LEAVE_CRITICAL_SECTION();

  return(E_OK);
}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_get_serial_device_number
         (
           OUT USIGN32 * pSerialDeviceNumber
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to get the serial device number of the FG's PROFIBUS controller

OUT:  pSerialDeviceNumber -> serial device number

Possible return values:
- E_OK
- E_IF_PAPI_NOT_INITIALIZED      -> API not initialized
- E_IF_INVALID_PARAMETER         -> invalid parameter
- E_IF_SOCKET_ERROR              -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  T_SOCKET_MESSAGE SocketMessage;
  INT16            RetVal;
  int              Offset;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  /* check input parameter ---------------------------------------------------------*/
  if (pSerialDeviceNumber == NULL)
  {
    return(E_IF_INVALID_PARAMETER);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_GET_SERIAL_DEVICE_NUMBER -------------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_get_serial_device_number",++FrameNumber);

  /* send message ---------------------------------------------------------------- */
  RetVal = socket_send_data(Socket,&SocketMessage,TRUE);

  if (RetVal == E_OK)
  {
    Offset = GET_DATA_OFFSET();
    *pSerialDeviceNumber = *((USIGN32*) &SocketMessage.Id.Data[Offset]);
  }

  LEAVE_CRITICAL_SECTION();

  return(RetVal);
}


/****************************************************************************/



FUNCTION GLOBAL INT16 CALL_CONV profi_get_last_error(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function returns the additional last error code of FG's PAPI INTERFACE-ERRORs
controller

Possible return values:
- additional last error code

- E_IF_PAPI_NOT_INITIALIZED      -> API not initialized
- E_IF_SOCKET_ERROR              -> TCP socket error

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  int              Result;
  T_SOCKET_MESSAGE SocketMessage;
  INT16            LastRetVal;
  int              Offset;

  FUNCTION_BODY

  if (!bPapiInitialized)
  {
    /* papi not initialized ------------------------------------------------------- */
    return(E_IF_PAPI_NOT_INITIALIZED);
  }

  ENTER_CRITICAL_SECTION();

  /* send PROFI_GET_LAST_ERROR -----------------------------------------------------*/
  /* build message header --------------------------------------------------------- */
  socket_build_request_message_header(&SocketMessage,"profi_get_last_error",++FrameNumber);

  /* send message ----------------------------------------------------------------- */
  Result = socket_send_data(Socket,&SocketMessage,TRUE);

  if (Result == E_OK)
  {
    Offset = GET_DATA_OFFSET();
    LastRetVal = *((INT16*) &SocketMessage.Id.Data[Offset]);

    LEAVE_CRITICAL_SECTION();
    return(LastRetVal);
  }
  else
  {
    LastRetVal = E_IF_SOCKET_ERROR;
  }

  LEAVE_CRITICAL_SECTION();

  return(LastRetVal);
}


/****************************************************************************/


#ifdef WIN32
FUNCTION BOOL WINAPI DllMain
         (
            IN HINSTANCE  hInstDLL,
            IN ULONG      Reason,
            IN LPVOID     pReserved
          )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

32Bit-DLL main function.
The function is called by MS-Windows during loading DLL.

Possible return values:
- TRUE

-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  FUNCTION_BODY

  switch (Reason)
  {
     case DLL_PROCESS_ATTACH:
     {
       bPapiInitialized = FALSE;
       FrameNumber      = 0;
       Socket           = 0;
       InitializeCriticalSection(&CriticalSection);
       break;
     }
     case DLL_THREAD_ATTACH:
     {
       break;
     }
     case DLL_THREAD_DETACH:
     {
       break;
     }
     case DLL_PROCESS_DETACH:
     {
       profi_end();
       bPapiInitialized = FALSE;
       DeleteCriticalSection(&CriticalSection);
       break;
     }
     default:
     {
       break;
     }
  }

  return TRUE;
}
#endif

#ifdef WIN32
#pragma check_stack
#endif
