/*****************************************************************************/
/*                                                                           */
/*                    SOFTING Industrial Automation GmbH                     */
/*                        Richard-Reitzner-Allee 6                           */
/*                              D-85540 Haar                                 */
/*                      Phone: (++49)-(0)89-45656-0                          */
/*                      Fax:   (++49)-(0)89-45656-399                        */
/*                                                                           */
/*         Copyright (C) SOFTING Industrial Automation GmbH 2008-2010        */
/*                            All Rights Reserved                            */
/*                                                                           */
/*****************************************************************************/
/*****************************************************************************/
/*                                                                           */
/*  Filename    : PAPISOCK.H                                                 */
/*  Version     : 5.45.0.00.release                                          */
/*  Date        : 30-November-2010                                           */
/*  Author      : SOFTING                                                    */
/*                                                                           */
/*  Description : This file contains the TYPES, STRUCTS and FUNCTION-        */
/*                DECLARATIONS of PAPISOCKET modul.                          */
/*                                                                           */
/*****************************************************************************/

#ifndef __PAPISOCK__
#define __PAPISOCK__

#pragma warning (disable : 4103)     /* used #pragma pack to change alignment */
#ifdef WIN32
#pragma pack(push,2)
#else
#pragma pack(2)
#endif
#pragma warning (default : 4103)

/*********************************************************
* CONSTANTS                                              *
**********************************************************/

#define MAX_PB_DATA_SIZE            512

#define PROTOCOL_TYP_LEN            10
#define FRAME_TYP_LEN               4
#define SERVICE_NAME_LEN            40
#define MAX_NUMBER_PARAMETER        10
#define MAX_FRAME_LEN               4096

#define MAJOR_VERSION               01
#define MINOR_VERSION               00
#define VERSION                     ((MAJOR_VERSION) << 8 | MINOR_VERSION)

/*********************************************************
* TYPEDEFS                                               *
**********************************************************/

#ifdef LINUX
typedef int SOCKET;
#endif

typedef struct _T_SOCKET_MESSAGE_HEADER
{
  USIGN8    protocol_typ[PROTOCOL_TYP_LEN];
  USIGN16   version;
  USIGN16   frame_number;
  USIGN8    frame_typ[FRAME_TYP_LEN];
  USIGN8    service_name[SERVICE_NAME_LEN];
  USIGN16   para_len[MAX_NUMBER_PARAMETER];
} T_SOCKET_MESSAGE_HEADER;


typedef struct _T_SOCKET_MESSAGE_RESULT
{
  INT16     error_code;
  INT16     add_detail;
} T_SOCKET_MESSAGE_RESULT;


typedef struct _T_SOCKET_MESSAGE
{
  T_SOCKET_MESSAGE_HEADER   Header;
  union
  {
    T_SOCKET_MESSAGE_RESULT Result;
    USIGN8                  Data[MAX_PB_DATA_SIZE + 22];
  } Id;
} T_SOCKET_MESSAGE, *PT_SOCKET_MESSAGE;


#define GET_DATA_OFFSET()       (sizeof(T_SOCKET_MESSAGE_RESULT))
#define GET_PARA_LEN_OFFSET()   ( 2 )


/*********************************************************
* FUNCTION DECLARATIONS                                  *
**********************************************************/

extern INT16 socket_init                         (char* pFgName, int Port, SOCKET* pSocket);
extern VOID  socket_build_request_message_header (T_SOCKET_MESSAGE* pSocketMessage, const CSTRING* ServiceString, USIGN16 FrameNumber);
extern INT16 socket_send_data                    (SOCKET Socket, T_SOCKET_MESSAGE* pSocketMessage, int bWaitForResponse);
extern VOID  socket_close                        (SOCKET Socket);



#pragma warning (disable : 4103)     /* used #pragma pack to reset alignment */
#ifdef WIN32
#pragma pack(pop)
#else
#pragma pack()
#endif
#pragma warning (default : 4103)


#endif

