/******************************************************************************
*                                                                             *
*                      SOFTING Industrial Automation GmbH                     *
*                          Richard-Reitzner-Allee 6                           *
*                                D-85540 Haar                                 *
*                        Phone: (++49)-(0)89-45656-0                          *
*                        Fax:   (++49)-(0)89-45656-399                        *
*                                                                             *
*           Copyright (C) SOFTING Industrial Automation GmbH 2009-2010        *
*                              All Rights Reserved                            *
*                                                                             *
*******************************************************************************


FILE_NAME               SOCKET.C

PROJECT_NAME            PROFIBUS

MODULE                  PAPI

COMPONENT_LIBRARY       PAPIClient Lib
                        PAPIClient DLL

AUTHOR                  SOFTING

VERSION                 5.45.0.00.release

DATE                    30-November-2010

STATUS                  finished


FUNCTIONAL_MODULE_DESCRIPTION




RELATED_DOCUMENTS
=============================================================================*/
#include "keywords.h"

INCLUDES

#ifdef WIN32
#include <winsock2.h>
#include <io.h>
#include <direct.h>
#elif defined (LINUX)
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#include <syslog.h>
#include <errno.h>
#include <string.h>
#include <malloc.h>
#else
#error no operating system defined !!!!
#endif

#include "pb_type.h"
#include "papisock.h"
#include "pb_err.h"


/****************************************************************************/

GLOBAL_DEFINES

/****************************************************************************/

LOCAL_DEFINES

/****************************************************************************/

EXPORT_TYPEDEFS

/****************************************************************************/

LOCAL_TYPEDEFS

/****************************************************************************/

FUNCTION_DECLARATIONS

/****************************************************************************/

EXPORT_DATA

/****************************************************************************/

IMPORT_DATA

/****************************************************************************/

LOCAL_DATA



/****************************************************************************/

#ifdef WIN32
#pragma check_stack (off)
#endif


/****************************************************************************/


FUNCTION LOCAL INT16 calculate_message_length
         (
           IN T_SOCKET_MESSAGE* pSocketMessage
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION
This function is used to calculate the total length of the message to send.

Possible return values:
- message length
----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  int i;
  int MessageLength;

  FUNCTION_BODY

  MessageLength = sizeof(pSocketMessage->Header);
  i = 0;

  while((pSocketMessage->Header.para_len[i] != 0) && (i < MAX_NUMBER_PARAMETER))
  {
    MessageLength += pSocketMessage->Header.para_len[i];
    i++;
  }

  return(MessageLength);
}


/****************************************************************************/


FUNCTION LOCAL INT16 socket_open
         (
           IN  CSTRING*  pHostName,
           IN  int       Port,
           OUT SOCKET*   pSocket
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION
The function is used to create a socket and establish a connection to the specified
port / socket.

Possible return values:
-  E_OK              -> OK
-  E_IF_SOCKET_ERROR -> socket error
----------------------------------------------------------------------------*/
{
  unsigned long      IPAddrInBinary;
  struct in_addr     InAddr;                              /* winsock2.h sys/socket.h */
  struct sockaddr_in Server;                              /* winsock2.h sys/socket.h */
  struct hostent*    pHost;                               /* winsock2.h sys/socket.h */

  if ((pHostName == NULL) || (Port == 0))
  {
    return(E_IF_SOCKET_ERROR);
  }

  *pSocket = 0;

#ifdef WIN32
  /* create a socket that is bound to a specific service provider ------------------ */
  if( (*pSocket = socket( AF_INET, SOCK_STREAM, 0 )) == INVALID_SOCKET )
  {
    return(E_IF_SOCKET_ERROR);
  }

  Server.sin_family = AF_INET;

  /* convert Ipv4 address (string) into a proper address for the IN_ADDR structure --*/
  if((IPAddrInBinary = inet_addr(pHostName)) == INADDR_NONE )
  {
    /* retrieve host information corresponding to a host name from the host database.*/
    if((pHost = gethostbyname(pHostName)) == NULL )
    {
      return(E_IF_SOCKET_ERROR);
    }
    memcpy( &Server.sin_addr, pHost->h_addr, pHost->h_length );
  }
  else
  {
    InAddr.S_un.S_addr = IPAddrInBinary;
    Server.sin_addr = InAddr;
  }

  Server.sin_port = htons( (unsigned short) Port );

  /* establish the connection to the specified socket ----------------------------- */
  if (connect (*pSocket, (struct sockaddr *) &Server, sizeof(Server)) != 0 )
  {
    return(E_IF_SOCKET_ERROR);
  }

#elif defined (LINUX)

  /* create a socket that is bound to a specific service provider ------------------ */
  if( (*pSocket = socket( AF_INET, SOCK_STREAM, 0 )) < 0 )
  {
    return(E_IF_SOCKET_ERROR);
  }

  Server.sin_family = AF_INET;

  /* convert Ipv4 address (string) into a proper address for the IN_ADDR structure --*/
  if((IPAddrInBinary = inet_addr(pHostName)) == -1 )
  {
    /* retrieve host information corresponding to a host name from the host database.*/
    if((pHost = gethostbyname(pHostName)) == NULL )
    {
      return(E_IF_SOCKET_ERROR);
    }
    memcpy( &Server.sin_addr, pHost->h_addr, pHost->h_length );
  }
  else
  {
    InAddr.s_addr = (unsigned int) IPAddrInBinary;
    Server.sin_addr = InAddr;
  }

  Server.sin_port = htons( (unsigned short) Port );

  /* establish the connection to the specified socket ----------------------------- */
  if (connect (*pSocket, (struct sockaddr *) &Server, sizeof(Server)) < 0 )
  {
    return(E_IF_SOCKET_ERROR);
  }
#else
#error no operating system defined !!!!
#endif

 return(0);
}


/****************************************************************************/


FUNCTION LOCAL INT16 receive_from_socket
         (
           IN SOCKET            Socket,
           IN T_SOCKET_MESSAGE* pSocketMessage
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION
The function is used to receive data from specified socket
port / socket.

Possible return values:
-  E_OK              -> OK
-  E_IF_SOCKET_ERROR -> socket error
----------------------------------------------------------------------------*/
{

  int   ReceivedLength, LengthToReceive;
  int   Result, i;
  char *pReceiveBuffer;
  char  bGotHeader = FALSE;
  INT16 RetVal     = E_OK;

  /* receive message header and data from socket ----------------------------------- */
  ReceivedLength  = 0;
  LengthToReceive = sizeof(pSocketMessage->Header);
  pReceiveBuffer  = (char*) pSocketMessage;

  while(ReceivedLength < LengthToReceive)
  {
    /* receive from socket --------------------------------------------------------- */
    Result = recv(Socket,(char*) &pReceiveBuffer[ReceivedLength], (LengthToReceive-ReceivedLength), 0);
#ifdef WIN32
    if(Result == SOCKET_ERROR)
    {
      return(E_IF_SOCKET_ERROR);
    }
#elif  defined (LINUX)
    if(Result == -1)
    {
      return(E_IF_SOCKET_ERROR);
    }
#else
#error no operating system defined !!!!
#endif
    else if (Result > 0)
    {
      ReceivedLength += Result;

      if ((ReceivedLength == sizeof(pSocketMessage->Header)) && (!bGotHeader))
      {
        /* Got complete header ----------------------------------------------------- */
        bGotHeader = TRUE;

        i=0;
        while((pSocketMessage->Header.para_len[i] != 0) && (i < MAX_NUMBER_PARAMETER))
        {
          LengthToReceive += pSocketMessage->Header.para_len[i];
          if(LengthToReceive > sizeof(T_SOCKET_MESSAGE))
          {
            return(E_IF_INVALID_DATA_SIZE);
          }
          i++;
        }
      }
    }
    else
    {
      /* connection is gracefully closed ------------------------------------------- */
      return(E_IF_SOCKET_ERROR);
    }
  }

  /* complete message received ----------------------------------------------------- */
  /* get function result ----------------------------------------------------------- */
  RetVal = pSocketMessage->Id.Result.error_code;

  return(RetVal);
}

/****************************************************************************/

FUNCTION GLOBAL INT16 socket_init
         (
           IN  char*     pFgName,
           IN  int       Port,
           OUT SOCKET*   pSocket
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION
This function is used to initialize a socket

Possible return values:
- E_OK                  -> OK
- E_IF_SOCKET_ERROR     -> Socket-Error
----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  int  Result = 0;

  FUNCTION_BODY

#ifdef WIN32
  unsigned short VersionRequested;
  WSADATA        WsaData;

  VersionRequested = MAKEWORD(2, 2);
  if(WSAStartup(VersionRequested, &WsaData))
  {
    return(E_IF_SOCKET_ERROR);
  }
#endif

  Result = socket_open ( pFgName, Port, pSocket);

  return(Result);
}


/****************************************************************************/

FUNCTION GLOBAL VOID socket_close
         (
           IN SOCKET Socket
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION
This function is used to close an existing socket.

Possible return values:
- NONE
----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  FUNCTION_BODY

#ifdef WIN32
  if (Socket)
  {
    closesocket (Socket);
    Socket = 0;
  }

  WSACleanup();

  return;

#elif defined (LINUX)

  if (Socket)
  {
    close (Socket);
    Socket = 0;
  }

  return;

#else
#error no operating system defined !!!!
#endif

}


/****************************************************************************/


FUNCTION GLOBAL VOID socket_build_request_message_header
         (
           IN T_SOCKET_MESSAGE* pSocketMessage,
           IN const CSTRING*    ServiceString,
           IN USIGN16           FrameNumber
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION
This function is used to build the message header to send data.

Possible return values:
- NONE
----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  FUNCTION_BODY

  memset(pSocketMessage->Header.para_len,0,sizeof(pSocketMessage->Header.para_len));

  strcpy(pSocketMessage->Header.protocol_typ,"PAPI");
  strcpy(pSocketMessage->Header.service_name, ServiceString);
  strcpy(pSocketMessage->Header.frame_typ, "REQ");
  pSocketMessage->Header.version      = VERSION;
  pSocketMessage->Header.frame_number = FrameNumber;
}


/* ================================================================================ */


FUNCTION GLOBAL INT16 socket_send_data
         (
           IN SOCKET            Socket,
           IN T_SOCKET_MESSAGE* pSocketMessage,
           IN BOOL              bWaitForResponse
         )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

The function is used to send data to a specified connected socket.

Possible return values:
-  E_OK              -> OK
-  E_IF_SOCKET_ERROR -> socket error
----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  INT16   RetVal = E_OK;
  USIGN8* SendBuffer;
  int     Result, SentLength, LengthToSend;

  FUNCTION_BODY


  /*---- SEND message to socket ----------------------------------------------*/
  SendBuffer = (USIGN8*) pSocketMessage;
  SentLength = 0;

  LengthToSend = calculate_message_length(pSocketMessage);

  while(SentLength < LengthToSend)
  {
    Result = send (Socket, &SendBuffer[SentLength], (LengthToSend - SentLength),0);
#ifdef WIN32
    if(Result == SOCKET_ERROR)
    {
      return(E_IF_SOCKET_ERROR);
    }
#elif  defined (LINUX)
    if(Result == -1)
    {
      return(E_IF_SOCKET_ERROR);
    }
#else
#error no operating system defined !!!!
#endif
    else
    {
      SentLength += Result;
    }
  }

  /*---- RECEIVE message from socket -----------------------------------------*/
  if (bWaitForResponse)
  {
    RetVal = receive_from_socket(Socket,pSocketMessage);
  }

  return(RetVal);
}

/****************************************************************************/

#ifdef WIN32
#pragma check_stack
#endif
