/******************************************************************************
*                                                                             *
*                      SOFTING Industrial Automation GmbH                     *
*                          Richard-Reitzner-Allee 6                           *
*                                D-85540 Haar                                 *
*                        Phone: (++49)-(0)89-45656-0                          *
*                        Fax:   (++49)-(0)89-45656-399                        *
*                                                                             *
*           Copyright (C) SOFTING Industrial Automation GmbH 1995-2010        *
*                              All Rights Reserved                            *
*                                                                             *
*******************************************************************************

FILE_NAME               FDLDEMO.C

PROJECT_NAME            PROFIBUS

MODULE                  FDLDEMO

COMPONENT_LIBRARY       --------

AUTHOR                  SOFTING

VERSION                 5.24.1

DATE                    30.07.2009

STATUS                  finished

FUNCTIONAL_MODULE_DESCRIPTION

FDL Demo Application
--------------------

CHANGE_NOTES

Date     Name      Change
------------------------------------------------------------------------------


RELATED_DOCUMENTS

============================================================================*/
#include "keywords.h"

INCLUDES

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef WIN32
 #include <conio.h>
 #include <windows.h>
 #include <errno.h>
 #include <dos.h>
#elif LINUX
 #include <unistd.h>
 #include <ctype.h>
 #include <errno.h>
#else
 #error : no operating system defined !!!
#endif

#include "pb_type.h"
#include "pb_conf.h"
#include "pb_if.h"
#include "pb_fmb.h"
#include "pb_fdl.h"
#include "pb_err.h"

LOCAL_DEFINES

/* --- default controller addresses --------------------------------------- */
#define BOARD_NUMBER                    0

/* --- station 5 or station 10 -------------------------------------------- */
#define STATION_5_TXT                 "5"
#define STATION_10_TXT                "10"


/* --- SAP's --------------------------------------------------------------- */
#define SND_SAP                     3
#define RCV_SAP                     4
#define RESP_SAP                   10
#define RCV_BRCT_SAP               63

#ifdef LINUX
#define stricmp(str1, str2)         strcasecmp(str1, str2)
#define getch getchar
#endif

LOCAL_TYPEDEFS

FUNCTION_DECLARATIONS

static VOID   print_version(VOID);
static VOID   print_papi_firmware_version(VOID);
extern short  CALL_CONV profi_snd_idle(void);

EXPORT_DATA

IMPORT_DATA

LOCAL_DATA

static short unsigned int      board_number;

static USIGN8                  station;

static T_PROFI_SERVICE_DESCR   sdb;
static USIGN8                  cnf_ind_buffer[512];
static USIGN8                  req_res_buffer[512];
static USIGN8                  loc_add;
static USIGN8                  rem_add;
static INT8                    sdn_invoke_id=0;
static INT8                    sda_invoke_id=0;
static INT8                    srd_invoke_id=0;
static USIGN8                  prio=LOW;



#ifdef LINUX

FUNCTION PUBLIC int kbhit (VOID)
/*------------------------------------------------------------------------*/
/* FUNCTIONAL_DESCRIPTION                                                 */
/*------------------------------------------------------------------------*/
/*                                                                        */
/*------------------------------------------------------------------------*/
{
   LOCAL_VARIABLES

   fd_set          rd_fds;
   struct timeval  timeout;

   FUNCTION_BODY

   FD_ZERO (&rd_fds);
   FD_SET (0, &rd_fds);

   timeout.tv_sec  = 0L;
   timeout.tv_usec = 1000L; /* wait 1 ms */

   if (select (1, &rd_fds, NULL, NULL, &timeout) > 0)
   {
      return (1);
   }

   return (0);
}

#endif


/* ------------------------------------------------------------------------- */
/* print interface error function                                            */
/* ------------------------------------------------------------------------- */

FUNCTION static VOID print_if_error
         (
           IN USIGN8           primitive,
           IN INT16            error
         )

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to print interface errors

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_EXCEPTION exception;
USIGN16     size = sizeof(T_EXCEPTION);

FUNCTION_BODY

if (primitive == REQ || primitive == RES)
  {
    printf("\n profi_snd_req_res failed\n");
  }
else if (primitive == IND || primitive == CON)
  {
    printf("\n profi_rcv_con_ind returns with error\n");
  }
else
  {
  }

switch (error)
{
  case E_IF_FATAL_ERROR:
       printf(" unrecoverable error on board     \n");
       while (profi_get_data(ID_EXCEPTION_IMAGE,
                             0,
                             (USIGN16 FAR*) &size,
                             (VOID FAR*) &exception
                            ) == E_IF_SERVICE_CONSTR_CONFLICT);

       printf(" task_id : %d (dez) %X (hex)\n",exception.task_id,exception.task_id);
       printf(" par1    : %d (dez) %X (hex)\n",exception.par1,exception.par1);
       printf(" par2    : %d (dez) %X (hex)\n",exception.par2,exception.par2);
       printf(" par3    : %d (dez) %X (hex)\n",exception.par3,exception.par3);
       break;

  case E_IF_INIT_INVALID_PARAMETER:    printf(" invalid initialize parameter     \n");
                                       break;
  case E_IF_LOADER_ERROR:              printf(" download firmware error          \n");
                                       break;
  case E_IF_NO_CNTRL_RES:              printf(" controller does not respond      \n");
                                       break;
  case E_IF_INVALID_CNTRL_TYPE_VERSION:printf(" inv. controller type or SW vers. \n");
                                       break;
  case E_IF_INVALID_LAYER:             printf(" invalid layer                    \n");
                                       break;
  case E_IF_INVALID_SERVICE:           printf(" invalid service identifier       \n");
                                       break;
  case E_IF_INVALID_PRIMITIVE:         printf(" invalid service primitive        \n");
                                       break;
  case E_IF_INVALID_DATA_SIZE:         printf(" not enough cmi data block memory \n");
                                       break;
  case E_IF_INVALID_COMM_REF:          printf(" invalid communication reference  \n");
                                       break;
  case E_IF_INVALID_CMI_CALL:          printf(" invalid CMI call                 \n");
                                       break;
  case E_IF_CMI_ERROR:                 printf(" error occured in CMI             \n");
                                       break;
  case E_IF_RESOURCE_UNAVAILABLE:      printf(" no resource available            \n");
                                       break;
  case E_IF_NO_PARALLEL_SERVICES:      printf(" no parallel services allowed     \n");
                                       break;
  case E_IF_SERVICE_CONSTR_CONFLICT:   printf(" serv. tempor. not executable     \n");
                                       break;
  case E_IF_SERVICE_NOT_SUPPORTED:     printf(" service not supported            \n");
                                       break;
  case E_IF_SERVICE_NOT_EXECUTABLE:    printf(" service not executable           \n");
                                       break;
  case E_IF_INVALID_VERSION:           printf(" no valid access to controller    \n");
                                       break;
  case E_IF_NO_CNTRL_PRESENT:          printf(" controller not available         \n");
                                       printf(" additonal error: 0x%2x \n",profi_get_last_error());
                                       break;
  case E_IF_INVALID_PARAMETER:         printf(" wrong parameter in REQ or RES    \n");
                                       break;
  case E_IF_INIT_FAILED:               printf(" initialize of PAPI failed\n");
                                       printf(" additonal error: %x \n",profi_get_last_error());
                                       break;
  case E_IF_EXIT_FAILED:               printf(" exit of PAPI failed     \n");
                                       printf(" additonal error: %x \n",profi_get_last_error());

                                       break;
#ifdef WIN32
  case E_IF_SOCKET_ERROR:              printf(" TCP socket error: (0x%04x)",WSAGetLastError());
                                       break;
  case E_IF_OS_ERROR:                  printf(" Operating system error: (0x%04x)",GetLastError());
                                       break;
#elif LINUX
  case E_IF_SOCKET_ERROR:              printf(" TCP socket error: (0x%04x)",errno);
                                       break;
  case E_IF_OS_ERROR:                  printf(" Operating system error: (0x%04x)", errno);
                                       break;
#endif
  default:                             printf(" unknown IF error   %d            \n",error); break;
}

printf("\n -> press any key \n"); getch();

return;
}



/* ------------------------------------------------------------------------- */
/* print data function                                                       */
/* ------------------------------------------------------------------------- */

FUNCTION static VOID print_data
(
    USIGN8 FAR  *buf,
    USIGN16     count
)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

Prints binary data in the following format:

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

USIGN16 i;

FUNCTION_BODY

 for (i=0;i<count;i++)
 {
   if ((i % 20)) printf(" %2x",buf[i]);
   else          if (i==0) printf(" data:  %2x",buf[i]);
                 else      printf("\n        %2x",buf[i]);
 }
 printf("\n");

 return;
}


/* ------------------------------------------------------------------------- */
/* FMB services                                                              */
/* - FMB_SET_CONFIGURATION                                                   */
/* - FMB_RESET                                                               */
/* - FMB_EXIT                                                                */
/* ------------------------------------------------------------------------- */


FUNCTION static INT16 fmb_set_configuration_req(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to set the system configuration

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FMB_SET_CONFIGURATION_REQ fmb_config_req;

FUNCTION_BODY

printf("\nFMB_SET_CONFIGURATION.req:");

fmb_config_req.fms_active                = PB_FALSE;
fmb_config_req.dp_active                 = PB_FALSE;
fmb_config_req.fdlif_active              = PB_TRUE;
fmb_config_req.sm7_active                = PB_FALSE;
fmb_config_req.fdl_evt_receiver          = FDLIF_USR;
fmb_config_req.data_buffer_length        = 255;

fmb_config_req.fdlif.send_req_credits = 20;
fmb_config_req.fdlif.srd_req_credits  = 20;
fmb_config_req.fdlif.receive_credits  = 20;
fmb_config_req.fdlif.max_no_resp_saps = 20;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FMB;
sdb.service    = FMB_SET_CONFIGURATION;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) &fmb_config_req,
                         (USIGN8) PB_TRUE
                        ));
}


FUNCTION static INT16 fmb_exit_req(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to exit the PROFIBUS PROTOCOL STACK


possible return values:

- == E_OK    -> no error
- != E_OK    -> error


-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES


FUNCTION_BODY

printf("\nFMB_EXIT.req:");

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FMB;
sdb.service    = FMB_EXIT;
sdb.primitive  = REQ;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                          NULL,
                         (USIGN8) PB_TRUE
                        ));
return(E_OK);
}


FUNCTION static INT16 fmb_reset_req(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to reset the PROFIBUS PROTOCOL STACK

possible return values:

- == E_OK    -> no error
- != E_OK    -> error


-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES


FUNCTION_BODY

printf("\nFMB_RESET.req:");

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FMB;
sdb.service    = FMB_RESET;
sdb.primitive  = REQ;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                          NULL,
                         (USIGN8) PB_TRUE
                        ));
}

/* ------------------------------------------------------------------------- */
/* FDLIF Management services:                                                */
/* - FDLIF_SET_BUSPARAMETER                                                  */
/* - FDLIF_READ_BUSPARAMETER                                                 */
/* - FDLIF_SAP_ACTIVATE                                                      */
/* - FDLIF_RSAP_ACTIVATE                                                     */
/* - FDLIF_SAP_DEACTIVATE                                                    */
/* - FDLIF_SAP_CHANGE_ACCESS                                                 */
/* - FDLIF_EXIT                                                              */
/* - FDLIF_EVENT                                                             */
/* ------------------------------------------------------------------------- */

FUNCTION extern INT16 fdlif_set_busparameter_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to set the busparameters

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SET_BUSPARAMETER_REQ bus_par_req;

FUNCTION_BODY

printf("\nFDLIF_SET_BUSPARAMETER.req:");

/* set default busparameter for baudrate 500KBit/s ------------------------ */
bus_par_req.loc_add         = loc_add;

bus_par_req.in_ring_desired = 255;
bus_par_req.baud_rate       = 4;
bus_par_req.tset            = 1;
bus_par_req.loc_segm        = 255;
bus_par_req.medium_red      = 0;
bus_par_req.tsl             = 3500;
bus_par_req.min_tsdr        = 500;
bus_par_req.max_tsdr        = 1000;
bus_par_req.tqui            = 0;
bus_par_req.ttr             = 100000;
bus_par_req.g               = 1;
bus_par_req.hsa             = 126;
bus_par_req.max_retry_limit = 1;


/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SET_BUSPARAMETER;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) &bus_par_req,
                         (USIGN8) PB_TRUE
                        ));
}



FUNCTION extern INT16 fdlif_read_busparameter_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to read the busparameters

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

printf("\nFDLIF_READ_BUSPARAMETER.req");

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_READ_BUSPARAMETER;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) NULL,
                         (USIGN8) PB_TRUE
                        ));
}


FUNCTION extern VOID fdlif_read_busparameter_con
         (
           IN T_FDLIF_READ_BUSPARAMETER_CNF FAR* cnf_ptr
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to receive the read busparameter confirmation

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

printf("\nFDLIF_READ_BUSPARAMETER.con(+)\n");

printf (" hsa:             %u\n", cnf_ptr->hsa);
printf (" loc_add:         %u\n", cnf_ptr->loc_add);
printf (" baud_rate:       %u\n", cnf_ptr->baud_rate);
printf (" medium_red:      %u\n", cnf_ptr->medium_red);
printf (" retry_ctr:       %u\n", cnf_ptr->max_retry_limit);
printf (" tsl:             %u\n", cnf_ptr->tsl);
printf (" tqui:            %u\n", cnf_ptr->tqui);
printf (" tset:            %u\n", cnf_ptr->tset);
printf (" min_tsdr:        %u\n", cnf_ptr->min_tsdr);
printf (" max_tsdr:        %u\n", cnf_ptr->max_tsdr);
printf (" ttr:             %lu\n",cnf_ptr->ttr);
printf (" g:               %u\n", cnf_ptr->g);
printf (" in_ring_desired: %u\n", cnf_ptr->in_ring_desired);

return;
}



FUNCTION extern INT16 fdlif_sap_activate_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to activate a SAP

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SAP_ACTIVATE_REQ sap;
int                      x;

FUNCTION_BODY

sap.sap_nr = 0;

printf("\nFDLIF_SAP_ACTIVATE.req:");

while(sap.sap_nr != SND_SAP &&
      sap.sap_nr != RCV_SAP &&
      sap.sap_nr != RCV_BRCT_SAP
     )
{
  printf("\nenter SAP-NO: %u=SND_SAP %u=RCV_SAP %u=BRCT_RCV_SAP ",
          SND_SAP,RCV_SAP,RCV_BRCT_SAP
        );
  scanf("%u",&x);
  sap.sap_nr = (USIGN8) x;
}

switch(sap.sap_nr)
{
  case SND_SAP:
  case RCV_SAP:
       /* --- set data block parameters ------------------------------------ */
       sap.max_l_sdu_length_req      = 244;
       sap.max_l_sdu_length_con_ind  = 245;
       sap.access_sap                = ALL;
       sap.access_station            = ALL;
       sap.sda                       = BOTH_ROLES;
       sap.sdn                       = BOTH_ROLES;
       sap.srd                       = BOTH_ROLES;
       sap.csrd                      = SERVICE_NOT_ACTIVATED;
       sap.data_mode                 = NORMAL_MODE;
       sap.credits                   = 2;
       break;

  case RCV_BRCT_SAP:
       /* --- set data block parameters ------------------------------------ */
       sap.max_l_sdu_length_req      = 244;
       sap.max_l_sdu_length_con_ind  = 245;
       sap.access_sap                = ALL;
       sap.access_station            = ALL;
       sap.sda                       = SERVICE_NOT_ACTIVATED;
       sap.sdn                       = RESPONDER;
       sap.srd                       = SERVICE_NOT_ACTIVATED;
       sap.csrd                      = SERVICE_NOT_ACTIVATED;
       sap.data_mode                 = NORMAL_MODE;
       sap.credits                   = 2;
       break;
}

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SAP_ACTIVATE;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) &sap,
                         (USIGN8) PB_TRUE
                        ));
}


FUNCTION extern INT16 fdlif_rsap_activate_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to activate a RSAP

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_RSAP_ACTIVATE_REQ rsap;

FUNCTION_BODY

printf("\nFDLIF_RSAP_ACTIVATE.req:  SAP-NO: %u",RESP_SAP);

/* --- set data block parameters ------------------------------------------- */
rsap.sap_nr                    = RESP_SAP;
rsap.max_l_sdu_length_req      = 244;
rsap.max_l_sdu_length_ind      = 245;
rsap.access_sap                = ALL;
rsap.access_station            = ALL;
rsap.data_mode                 = NORMAL_MODE;
rsap.credits                   = 2;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_RSAP_ACTIVATE;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) &rsap,
                         (USIGN8) PB_TRUE
                        ));
}




FUNCTION extern INT16 fdlif_sap_deactivate_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to deactivate a RSAP

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SAP_DEACTIVATE_REQ sap;
int                        x;

FUNCTION_BODY

printf("\nFDLIF_SAP_DEACTIVATE.req:");

/* --- set data block parameters ------------------------------------------- */
printf("\nenter SAP-NO: %u=SND_SAP %u=RCV_SAP %u=RESP_SAP %u=BRCT_RCV_SAP ",
       SND_SAP,RCV_SAP,RESP_SAP,RCV_BRCT_SAP);

scanf("%u",&x);
sap.sap_nr = (USIGN8) x;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SAP_DEACTIVATE;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) &sap,
                         (USIGN8) PB_TRUE
                        ));
}





FUNCTION extern INT16 fdlif_sap_change_access_req
                (
                  IN USIGN8 access_sap
                )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to change SAP access

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SAP_CHANGE_REQ  sap;

FUNCTION_BODY

printf("\nFDLIF_SAP_CHANGE_ACCESS.req:");

/* --- set data block parameters ------------------------------------------- */
sap.sap_nr         = RCV_SAP;
sap.access_sap     = access_sap;
sap.access_station = ALL;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SAP_CHANGE_ACCESS;
sdb.primitive  = REQ;
sdb.invoke_id  = 0;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) &sap,
                         (USIGN8) PB_TRUE
                        ));
}


FUNCTION static INT16 fdlif_exit_req(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to exit the FDLIF


possible return values:

- == E_OK    -> no error
- != E_OK    -> error


-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES


FUNCTION_BODY

printf("\nFDLIF_EXIT.req:");

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_EXIT;
sdb.primitive  = REQ;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                          NULL,
                         (USIGN8) PB_TRUE
                        ));
return(E_OK);
}

FUNCTION extern VOID fdlif_event_ind
         (
           IN T_FDLIF_EVENT_IND FAR* ind_ptr
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to receive the FDLIF event

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

printf("\nFDLIF_EVENT.ind: %u",ind_ptr->event);

return;
}


/* ------------------------------------------------------------------------- */
/* FDLIF Data-Transfer services:                                             */
/* - FDLIF_SDN                                                               */
/* - FDLIF_SDN_BROADCAST                                                     */
/* - FDLIF_SDA                                                               */
/* - FDLIF_SRD                                                               */
/* - FDLIF_REPLY_UPDATE                                                      */
/* - FDLIF_REPLY_UPDATE_MULTIPLE                                             */
/* ------------------------------------------------------------------------- */


FUNCTION extern INT16 fdlif_sdn_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to send a SDN request

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SDN_SDA_SRD_REQ *req = (T_FDLIF_SDN_SDA_SRD_REQ*) req_res_buffer;
USIGN8                  *data_ptr;

FUNCTION_BODY

/* --- set data block parameters ------------------------------------------- */
printf("\nFDLIF_SDN.req:");

data_ptr = (USIGN8*) (req+1);

/* --- set data block parameters ------------------------------------------- */
req->ssap      = SND_SAP;
req->dsap      = RCV_SAP;
req->rem_add   = rem_add;
req->priority  = prio = (USIGN8) ((prio+1) % 2);

data_ptr[0] = 0x12;
data_ptr[1] = 0x34;
data_ptr[2] = 0x56;
data_ptr[3] = 0x78;
data_ptr[4] = 0x9A;
data_ptr[5] = 0xAB;
data_ptr[6] = 0xCD;
data_ptr[7] = 0xEF;
data_ptr[8] = 0xFF;

req->length = 9;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SDN;
sdb.primitive  = REQ;
sdb.invoke_id  = sdn_invoke_id++;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) req,
                         (USIGN8) PB_TRUE
                        ));
}

FUNCTION extern INT16 fdlif_sdn_brct_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to send a SDN as Broadcast request

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SDN_SDA_SRD_REQ *req = (T_FDLIF_SDN_SDA_SRD_REQ*) req_res_buffer;
USIGN8                  *data_ptr;

FUNCTION_BODY

/* --- set data block parameters ------------------------------------------- */
printf("\nFDLIF_SDN.req (BRCT):");

data_ptr = (USIGN8*) (req+1);

/* --- set data block parameters ------------------------------------------- */
req->ssap      = SND_SAP;
req->dsap      = RCV_BRCT_SAP;
req->rem_add   = 127;                                            /* rem_add; */
req->priority  = prio = (USIGN8) ((prio+1) % 2);

data_ptr[0] = FDLIF_SDN;
data_ptr[1] = (USIGN8) sdn_invoke_id;
req->length = 2;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SDN;
sdb.primitive  = REQ;
sdb.invoke_id  = sdn_invoke_id++;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) req,
                         (USIGN8) PB_TRUE
                        ));
}

FUNCTION extern VOID fdlif_sdn_ind
         (
           IN INT8                        invoke_id,
           IN T_FDLIF_SDN_SDA_SRD_REQ FAR *ind_ptr
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to receive a SDN indication

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

USIGN8                  *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_SDN.ind:\n");

data_ptr = (USIGN8*) (ind_ptr+1);

printf (" invoke_id:   0x%x\n",invoke_id);
printf (" ssap:        %u\n",ind_ptr->ssap);
printf (" dsap:        %u\n",ind_ptr->dsap);
printf (" rem add:     %u\n",ind_ptr->rem_add);
printf (" prio:        %u\n",ind_ptr->priority);
printf (" data lenght: %u\n",ind_ptr->length);
print_data (data_ptr,ind_ptr->length);

return;
}


FUNCTION extern INT16 fdlif_sda_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to send a SDA request

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SDN_SDA_SRD_REQ *req = (T_FDLIF_SDN_SDA_SRD_REQ*) req_res_buffer;
USIGN8                  *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_SDA.req:");

/* --- set data block parameters ------------------------------------------- */
data_ptr = (USIGN8*) (req+1);

req->ssap      = SND_SAP;
req->dsap      = RCV_SAP;
req->rem_add   = rem_add;
req->priority  = prio = (USIGN8) ((prio+1) % 2);

data_ptr[0] = 0x12;
data_ptr[1] = 0x34;
data_ptr[2] = 0x56;
data_ptr[3] = 0x78;
data_ptr[4] = 0x9A;
data_ptr[5] = 0xAB;
data_ptr[6] = 0xCD;
data_ptr[7] = 0xEF;
data_ptr[8] = 0xFF;

req->length = 9;
/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SDA;
sdb.primitive  = REQ;
sdb.invoke_id  = sda_invoke_id++;

return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) req,
                         (USIGN8) PB_TRUE
                        ));
}


FUNCTION extern VOID fdlif_sda_ind
         (
           IN INT8                        invoke_id,
           IN T_FDLIF_SDN_SDA_SRD_REQ FAR *ind_ptr
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to receive a SDA indication

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

USIGN8                  *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_SDA.ind:\n");

data_ptr = (USIGN8*) (ind_ptr+1);

printf (" invoke_id:   0x%x \n",invoke_id);
printf (" ssap:        %u\n",ind_ptr->ssap);
printf (" dsap:        %u\n",ind_ptr->dsap);
printf (" rem add:     %u\n",ind_ptr->rem_add);
printf (" prio:        %u\n",ind_ptr->priority);
printf (" data lenght: %u\n",ind_ptr->length);
print_data (data_ptr,ind_ptr->length);

return;
}



FUNCTION extern INT16 fdlif_srd_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to send a SRD request

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_SDN_SDA_SRD_REQ *req = (T_FDLIF_SDN_SDA_SRD_REQ*) req_res_buffer;
USIGN8                  *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_SRD.req:");

/* --- set data block parameters ------------------------------------------- */
data_ptr = (USIGN8*) (req+1);

req->ssap      = SND_SAP;
req->dsap      = RESP_SAP;
req->rem_add   = rem_add;
req->priority  = prio = (USIGN8) ((prio+1) % 2);

data_ptr[0] = 0x12;
data_ptr[1] = 0x34;
data_ptr[2] = 0x56;
data_ptr[3] = 0x78;
data_ptr[4] = 0x9A;
data_ptr[5] = 0xAB;
data_ptr[6] = 0xCD;
data_ptr[7] = 0xEF;
data_ptr[8] = 0xFF;

req->length = 9;
/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_SRD;
sdb.primitive  = REQ;
sdb.invoke_id  = srd_invoke_id++;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) req,
                         (USIGN8) PB_TRUE
                        ));
}


FUNCTION extern VOID fdlif_srd_ind
         (
           IN INT8                        invoke_id,
           IN T_FDLIF_SDN_SDA_SRD_REQ FAR *ind_ptr
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to receive a SRD indication

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

USIGN8  FAR *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_SRD.ind:\n");

data_ptr = (USIGN8 FAR*) (ind_ptr+1);

printf (" invoke_id:   0x%x\n",invoke_id);
printf (" ssap:        %u\n",ind_ptr->ssap);
printf (" dsap:        %u\n",ind_ptr->dsap);
printf (" rem add:     %u\n",ind_ptr->rem_add);
printf (" prio:        %u\n",ind_ptr->priority);
printf (" status:      %u\n",ind_ptr->status);
printf (" data lenght: %u\n",ind_ptr->length);
print_data (data_ptr,ind_ptr->length);

return;
}


FUNCTION extern VOID fdlif_srd_con
         (
           IN INT16               result,
           IN INT8                invoke_id,
           IN T_FDLIF_SRD_CNF FAR *cnf_ptr
         )

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to receive a SRD confirmation

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

USIGN8  FAR *data_ptr;

FUNCTION_BODY

if (result == POS)
{
  printf("\n FDLIF_SRD.con(+)\n");
}
else
{
  printf("\n FDLIF_SRD.con(-)\n");
}

printf (" invoke_id:    0x%x\n",invoke_id);
printf (" status:       0x%x\n",cnf_ptr->status);
printf (" data lenght:  %u\n"  ,cnf_ptr->length);
if (cnf_ptr->length > 0)
{
     data_ptr = (USIGN8 FAR*) (cnf_ptr + 1);
     print_data (data_ptr, cnf_ptr->length);
}

return;
}


FUNCTION extern INT16 fdlif_reply_update_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to send a REPLY-UPDATE request

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_RUP_REQ         *req = (T_FDLIF_RUP_REQ*) req_res_buffer;
USIGN8                  *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_REPLY_UPDATE.req:");

/* --- set data block parameters ------------------------------------------- */
data_ptr = (USIGN8*) (req+1);

req->sap_nr    = RESP_SAP;
req->priority  = prio = (USIGN8) ((prio+1) % 2);

data_ptr[0] = FDLIF_REPLY_UPDATE;
data_ptr[1] = (USIGN8) srd_invoke_id;
req->length = 2;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_REPLY_UPDATE;
sdb.primitive  = REQ;
sdb.invoke_id  = srd_invoke_id++;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) req,
                         (USIGN8) PB_TRUE
                        ));
}



FUNCTION extern INT16 fdlif_reply_update_mult_req(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to send a REPLY-UPDATE-MULTIPLE request

possible return values:

- == E_OK    -> no error
- != E_OK    -> error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

T_FDLIF_RUP_REQ         *req = (T_FDLIF_RUP_REQ*) req_res_buffer;
USIGN8                  *data_ptr;

FUNCTION_BODY

printf("\nFDLIF_REPLY_UPDATE_MULTIPLE.req:");

/* --- set data block parameters ------------------------------------------- */
data_ptr = (USIGN8*) (req+1);

req->sap_nr    = RESP_SAP;
req->priority  = prio = (USIGN8) ((prio+1) % 2);

data_ptr[0] = FDLIF_REPLY_UPDATE_MULTIPLE;
data_ptr[1] = (USIGN8) srd_invoke_id;
req->length = 2;

/* --- set parameter block parameters -------------------------------------- */
sdb.comm_ref   = 0;
sdb.layer      = FDLIF;
sdb.service    = FDLIF_REPLY_UPDATE_MULTIPLE;
sdb.primitive  = REQ;
sdb.invoke_id  = srd_invoke_id++;


return(profi_snd_req_res((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                         (VOID FAR *) req,
                         (USIGN8) PB_TRUE
                        ));
}





FUNCTION static VOID wait_for_cnf_ind(VOID)

/*-----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

This function is used to receive an indication or confirmation from
the communication interface

possible return values:
- NONE

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

USIGN16                      buffer_len;
USIGN16                      ret_val;
T_FDLIF_ERROR            FAR *fdlif_error = (T_FDLIF_ERROR FAR*) cnf_ind_buffer;
T_ERROR                  FAR *fmb_error   = (T_ERROR       FAR*) cnf_ind_buffer;

USIGN8 c = '\0';


FUNCTION_BODY


profi_snd_idle ();

buffer_len = 512;
ret_val = profi_rcv_con_ind((T_PROFI_SERVICE_DESCR FAR*) &sdb,
                            (VOID                  FAR*) cnf_ind_buffer,
                            (USIGN16               FAR*) &buffer_len);

Sleep(1);

if (ret_val == CON_IND_RECEIVED)
{
     switch(sdb.layer)
     {
        case FMB_USR:
             switch(sdb.service)
             {
               case FMB_SET_CONFIGURATION:
                    if (sdb.result == POS) printf("\nFMB_SET_CONFIGURATION.con(+)");
                    else                   printf("\nFMB_SET_CONFIGURATION.con(-) 0x%X",fmb_error->class_code);
                    break;

               case FMB_EXIT:
                    if (sdb.result == POS) printf("\nFMB_EXIT.con(+)");
                    else                   printf("\nFMB_EXIT.con(-) 0x%X",fmb_error->class_code);
                    break;

               case FMB_RESET:
                    if (sdb.result == POS) printf("\nFMB_RESET.con(+)");
                    else                   printf("\nFMB_RESET.con(-) 0x%X",fmb_error->class_code);
                    break;

               default:


                    printf("\nnot supported FMB service received\n");
                    printf("service: %d primitive: %d\n",sdb.service,sdb.primitive);
             }
             break;


        case FDLIF_USR:
             switch(sdb.service)
             {
               case FDLIF_EVENT:
                    fdlif_event_ind((T_FDLIF_EVENT_IND FAR*) cnf_ind_buffer);
                    break;

               case FDLIF_SET_BUSPARAMETER:
                    if (sdb.result == POS) printf("\nFDLIF_SET_BUSPARAMTER.con(+)");
                    else                   printf("\nFDLIF_SET_BUSPARAMTER.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_READ_BUSPARAMETER:
                    if (sdb.result == POS) fdlif_read_busparameter_con((T_FDLIF_READ_BUSPARAMETER_CNF FAR*) cnf_ind_buffer);
                    else                   printf("\nFDLIF_READ_BUSPARAMTER.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_SAP_ACTIVATE:
                    if (sdb.result == POS) printf("\nFDLIF_SAP_ACTIVATE.con(+)");
                    else                   printf("\nFDLIF_SAP_ACTIVATE.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_RSAP_ACTIVATE:
                    if (sdb.result == POS) printf("\nFDLIF_RSAP_ACTIVATE.con(+)");
                    else                   printf("\nFDLIF_RSAP_ACTIVATE.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_SAP_DEACTIVATE:
                    if (sdb.result == POS) printf("\nFDLIF_SAP_DEACTIVATE.con(+)");
                    else                   printf("\nFDLIF_SAP_DEACTIVATE.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_SAP_CHANGE_ACCESS:
                    if (sdb.result == POS) printf("\nFDLIF_SAP_CHANGE_ACCESS.con(+)");
                    else                   printf("\nFDLIF_SAP_CHANGE_ACCESS.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_REPLY_UPDATE:
                    if (sdb.result == POS) printf("\nFDLIF_REPLY_UPDATE.con(+)");
                    else                   printf("\nFDLIF_REPLY_UPDATE.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_REPLY_UPDATE_MULTIPLE:
                    if (sdb.result == POS) printf("\nFDLIF_REPLY_UPDATE_MULTIPLE.con(+)");
                    else                   printf("\nFDLIF_REPLY_UPDATE_MULTIPLE.con(-) 0x%X",fdlif_error->result);
                    break;

               case FDLIF_SDN:
                    if (sdb.primitive == IND)
                    {
                      fdlif_sdn_ind(sdb.invoke_id,(T_FDLIF_SDN_SDA_SRD_REQ FAR*) cnf_ind_buffer);
                    }
                    else
                    {
                      if (sdb.result == POS) printf("\nFDLIF_SDN.con(+)");
                      else                   printf("\nFDLIF_SDN.con(-) 0x%X",fdlif_error->result);
//                      fdlif_sdn_req();
                    }
                    break;

               case FDLIF_SDA:
                    if (sdb.primitive == IND)
                    {
                      fdlif_sda_ind(sdb.invoke_id,(T_FDLIF_SDN_SDA_SRD_REQ FAR*) cnf_ind_buffer);
                    }
                    else
                    {
                      if (sdb.result == POS) printf("\nFDLIF_SDA.con(+)");
                      else                   printf("\nFDLIF_SDA.con(-) 0x%X",fdlif_error->result);
//                      fdlif_sda_req();
                    }
                    break;

               case FDLIF_SRD:
                    if (sdb.primitive == IND)
                    {
                      fdlif_srd_ind(sdb.invoke_id,(T_FDLIF_SDN_SDA_SRD_REQ FAR*) cnf_ind_buffer);
                    }
                    else
                    {
                      fdlif_srd_con(sdb.result,sdb.invoke_id,(T_FDLIF_SRD_CNF FAR*) cnf_ind_buffer);
//                      fdlif_srd_req();
                    }
                    break;

               case FDLIF_EXIT:
                    if (sdb.result == POS) printf("\nFDLIF_EXIT.con(+)");
                    else                   printf("\nFDLIF_EXIT.con(-) 0x%X",fdlif_error->result);
                    break;

                default:
                    printf("\nnot supported FDLIF service received\n");
                    printf("service: %d primitive: %d\n",sdb.service,sdb.primitive);
             }
             break;

        default:
            printf("\nnot supported layer service received\n");
            printf("layer: %d service: %d primitive: %d\n",sdb.layer,sdb.service,sdb.primitive);
            break;
     }
}
else
{
  if (ret_val != NO_CON_IND_RECEIVED) print_if_error(IND,ret_val);
}
return;
}



FUNCTION static VOID schedule(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

the scheduler waits for keyboard inputs of the user

- with the demo application the user can send the following FDL services

'A' : FMB set configuration      'K' : FDLIF send SDN
'B' : FDLIF set busparameter     'L' : FDLIF send SDA
'C' : FDLIF read busparameter    'M' : FDLIF send SRD
'D' : FDLIF SAP activate         'N' : FDLIF reply update
'E' : FDLIF RSAP activate        'O' : FDLIF reply update multiple
'F' : FDLIF SAP deactivate
'G' : FDLIF SAP chance access
'H' : FDLIF exit
'I' : FMB reset
'J' : FMB exit
'X' : EXIT program


-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

INT16 ret_val;
USIGN8 c = '\0';

FUNCTION_BODY


 while (toupper(c) != 'X')
 {
   ret_val = E_OK;

   printf("\n                PROFIBUS FDL DEMO STATION %u \n",loc_add);
   printf("                ----------------------------\n");
   printf("\n");
   printf("  'A' : FMB Set Configuration      'K' : FDLIF Send SDN\n");
   printf("  'B' : FDLIF Set Busparameter     'L' : FDLIF Send SDN (BRCT)\n");
   printf("  'C' : FDLIF Read Busparameter    'M' : FDLIF Send SDA\n");
   printf("  'D' : FDLIF SAP Activate         'N' : FDLIF Send SRD\n");
   printf("  'E' : FDLIF RSAP Activate        'O' : FDLIF Reply Update\n");
   printf("  'F' : FDLIF SAP Deactivate       'P' : FDLIF Reply Update multiple\n");
   printf("  'G' : FDLIF SAP Change Access\n");
   printf("  'H' : FDLIF Exit\n");
   printf("  'I' : FMB Reset\n");
   printf("  'J' : FMB Exit\n\n");
   printf("  'X' : EXIT Program\n");
   printf("\n");
   printf("\n");

   while (!kbhit()) wait_for_cnf_ind();
   c = (USIGN8) getch();

   switch (toupper(c))
   {
     case 'A':
          ret_val = fmb_set_configuration_req();
          break;

     case 'B':
          ret_val = fdlif_set_busparameter_req();
          break;

     case 'C':
          ret_val = fdlif_read_busparameter_req();
          break;

     case 'D':
          ret_val = fdlif_sap_activate_req();
          break;

     case 'E':
          ret_val = fdlif_rsap_activate_req();
          break;

     case 'F':
          ret_val = fdlif_sap_deactivate_req();
          break;

     case 'G':
          ret_val = fdlif_sap_change_access_req(SND_SAP);
          break;

     case 'H':
          ret_val = fdlif_exit_req();
          break;

     case 'I':
          ret_val = fmb_reset_req();
          break;

     case 'J':
          ret_val = fmb_exit_req();
          break;

     case 'K':
          ret_val = fdlif_sdn_req();
          break;

     case 'L':
          ret_val = fdlif_sdn_brct_req();
          break;

     case 'M':
          ret_val = fdlif_sda_req();
          break;

     case 'N':
          ret_val = fdlif_srd_req();
          break;

     case 'O':
          ret_val = fdlif_reply_update_req();
          break;

     case 'P':
          ret_val = fdlif_reply_update_mult_req();
          break;

     case 'X':
          profi_end();
          exit(0);

     default:
          ret_val = E_OK;
          break;
   }
   if (ret_val != E_OK) print_if_error(REQ,ret_val);

 }
 return;
}



FUNCTION static INT16 init_profi(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

this function is used to initialize the communication interface

possible return values:

- == E_OK    -> no error

-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

INT16  ret_val = E_OK;

FUNCTION_BODY

ret_val = profi_init((USIGN8) board_number,0,0);


if (ret_val == E_OK)
{
  /* --- print version------------------------------------------------------ */
  print_version();
  print_papi_firmware_version();
  printf("\n\n press any key");
  getch();
}

return(ret_val);
}


/* ------------------------------------------------------------------------- */
/* Functions for initializing the program and program parameters             */
/* - print program version                                                   */
/* - print program usage                                                     */
/* - check command line parameters                                           */
/* - set default options                                                     */
/* ------------------------------------------------------------------------- */


FUNCTION static VOID print_version(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

prints program version on screen

possible return values:

- NONE
-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf ("\n  PROFIBUS FDL-Demo V 5.24 (C) 1995-2010 Softing Industrial Automation GmbH\n\n");
  printf ("                      Softing Industrial Automation GmbH\n");
  printf ("                          Richard-Reitzner-Allee 6\n");
  printf ("                             D-85540 Haar\n\n");
  printf ("                     Phone: (++49) (0) 89 / 45 65 6 - 0\n");
  printf ("                     Fax  : (++49) (0) 89 / 45 65 6 - 399\n\n");

}

FUNCTION static VOID print_papi_firmware_version(VOID)

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

prints program version on screen

possible return values:

- NONE
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  CSTRING   papi_version[VERSION_STRING_LENGTH];
  CSTRING   fw_version[VERSION_STRING_LENGTH];
  USIGN32   serial_device_number;

  FUNCTION_BODY

  profi_get_versions((CSTRING FAR*) papi_version,(CSTRING FAR*) fw_version);
  profi_get_serial_device_number((USIGN32 FAR*)&serial_device_number);

  printf("\n Serial Device Number: %lu",serial_device_number);
  printf("\n PAPI Version:         %s" ,papi_version);
  printf("\n FW Version:           %s" ,fw_version);
}


FUNCTION static VOID print_usage
  (
  IN    VOID
  )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

prints program version and usage on screen

possible return values:

- NONE
-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

  printf ("Usage  : fdldemo [options] [station]\n\n");

  printf ("options: \n");
  printf ("-b<add>  = Board Number          (default: 0\n");
  printf ("-h | -?  = display this help information\n\n");

  printf ("station: \n");
  printf ("5        = station address 5     (client)\n");
  printf ("10       = station address 10    (server)\n\n");

} /* print_usage */



FUNCTION static VOID check_options
  (
  IN    char    options []
  )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

check command line parameters for valid options and TS5/TS10

valid options:
    -b<BOARD_NUMBER>             only WIN32
    -a<DPR_ADDRESS>              only WIN16
    -i<IO_ADDRESS>               only WIN16
    -h | -?                      print usage
possible return values:

- NONE
-----------------------------------------------------------------------------*/
{
  LOCAL_VARIABLES

  char *token_ptr;

  FUNCTION_BODY

  token_ptr = strtok (options, " \t\r");

  if (token_ptr != NULL)
  {
    if (strncmp (token_ptr, "-b", 2) == 0)
    {
      if (sscanf (token_ptr, "-b%hi", &board_number) == 1)
      {
        if(!((board_number >= 0 ) && (board_number <= 9 )))
        {
          board_number = BOARD_NUMBER;
        }
      }
      else
      {
        board_number = BOARD_NUMBER;
      }
    }
    else if ( stricmp (token_ptr, STATION_5_TXT) == 0 )
    {
      loc_add = 5;
      rem_add = 10;
    }
    else if ( stricmp (token_ptr, STATION_10_TXT) == 0 )
    {
      loc_add = 10;
      rem_add = 5;
    }
    else if ( (strcmp (token_ptr, "-?") == 0) ||
              (strcmp (token_ptr, "-h") == 0) )
    {
      print_version ();
      print_usage ();

      exit (0);
    }
    else
    {
      printf ("error: unknown command line parameter \"%s\" \n\n", token_ptr);
      print_usage ();

      exit (-1);
    }

    token_ptr = strtok (NULL, " \t\r");

  } /* if token_ptr */
}


FUNCTION LOCAL VOID set_default_options
  (
  IN    VOID
  )
/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

set default options for command line parameters

possible return values:

- NONE
-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

FUNCTION_BODY

  board_number      = BOARD_NUMBER;

  loc_add = 5;
  rem_add = 10;

}



/* ------------------------------------------------------------------------- */
/* main program                                                              */
/* ------------------------------------------------------------------------- */

FUNCTION extern VOID main
        (
          IN  INT16  argc,
          IN  USIGN8 *argv[]
        )

/*----------------------------------------------------------------------------
FUNCTIONAL_DESCRIPTION

main procedure of the demo application

possible return values:

- NONE
-----------------------------------------------------------------------------*/
{
LOCAL_VARIABLES

  int       i;
  INT16     ret_val;

FUNCTION_BODY

  set_default_options ();                      /* program default parameters */

/* -- check command line --------------------------------------------------- */
  for (i = 1; i < argc; i++)
  {
    check_options (argv [i]);
  }

  print_version();

  printf("Settings:\n");
  printf("acting as %s \n", (loc_add==5) ? STATION_5_TXT : STATION_10_TXT);

  /* initialize the communication interface ---------------------------------- */
  if ((ret_val = init_profi()) == E_OK)
  {
     printf("\nInitialization successfully\n");

     /* schedule the application and communication --------------------------- */
     schedule ();

     profi_end();
     exit(0);
  }
  else
  {
     printf("\nInitialization failed\n");
     print_if_error(0xFF,ret_val);
     exit(1);
  }

exit(0);
}

